@@ -1,7 +1,7 @@
 /* Evolution calendar - weather backend source class for parsing
- * 	CCF (coded cities forecast) formatted NWS reports
+ *	CCF (coded cities forecast) formatted NWS reports
  *
- * Copyright (C) 2005 Novell, Inc (www.novell.com)
+ * Copyright (C) 1999-2008 Novell, Inc. (www.novell.com)
  *
  * Authors: David Trowbridge <trowbrds@cs.colorado.edu>
  *
@@ -26,18 +26,14 @@
 #include <string.h>
 #include <stdlib.h>
 
-#include <libxml/parser.h>
-#include <libxml/tree.h>
-
 #include <glib/gi18n-lib.h>
 
-#include <gconf/gconf-client.h>
-
-#include "libedataserver/e-xml-utils.h"
-
 #include "e-weather-source-ccf.h"
 
-#define DATA_SIZE 5000
+#define GWEATHER_I_KNOW_THIS_IS_UNSTABLE
+#include <libgweather/weather.h>
+#include <libgweather/gweather-xml.h>
+#undef GWEATHER_I_KNOW_THIS_IS_UNSTABLE
 
 #ifdef G_OS_WIN32
 
@@ -58,91 +54,103 @@
 #define strtok_r(s,sep,lasts) (*(lasts)=strtok((s),(sep)))
 #endif
 
-static gchar *
-parse_for_url (char *code, char *name, xmlNode *parent)
+struct search_struct
 {
-	xmlNode *child;
-	if (parent->type == XML_ELEMENT_NODE) {
-		if (strcmp ((char*)parent->name, "location") == 0) {
-			child = parent->children;
-			g_assert (child->type == XML_TEXT_NODE);
-			if (strcmp ((char*)child->content, name) == 0) {
-				xmlAttr *attr;
-				gchar *url = NULL;
-				for (attr = parent->properties; attr; attr = attr->next) {
-					if (strcmp ((char*)attr->name, "code") == 0) {
-						if (strcmp ((char*)attr->children->content, code) != 0)
-							return NULL;
-					}
-					if (strcmp ((char*)attr->name, "url") == 0)
-						url = (char*)attr->children->content;
-				}
-				return g_strdup (url);
-			}
-			return NULL;
-		} else {
-			for (child = parent->children; child; child = child->next) {
-				gchar *url = parse_for_url (code, name, child);
-				if (url)
-					return url;
-			}
-		}
+	const gchar *code;
+	const gchar *name;
+	gboolean is_old;
+	WeatherLocation *location;
+};
+
+static gboolean
+find_location_func (GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *node, gpointer data)
+{
+	WeatherLocation *wl = NULL;
+	struct search_struct *search = (struct search_struct *)data;
+
+	gtk_tree_model_get (model, node, GWEATHER_XML_COL_POINTER, &wl, -1);
+	if (!wl || !wl->name || !wl->code || !search || search->location)
+		return FALSE;
+
+	if (((!strcmp (wl->code, search->code)) || (search->is_old && !strcmp (wl->code + 1, search->code))) &&
+	     (!strcmp (wl->name, search->name))) {
+		search->location = weather_location_clone (wl);
+		return TRUE;
 	}
-	return NULL;
+
+	return FALSE;
 }
 
-static void
-find_station_url (gchar *station, EWeatherSourceCCF *source)
+static WeatherLocation *
+find_location (const gchar *code_name, gboolean is_old)
 {
-	xmlDoc *doc;
-	xmlNode *root;
-	gchar **sstation;
-	gchar *url;
-	gchar *filename;
+	GtkTreeModel *model;
+	gchar **ids;
+	struct search_struct search;
 
-	sstation = g_strsplit (station, "/", 2);
+	search.location = NULL;
 
-#ifndef G_OS_WIN32
-	filename = g_strdup (WEATHER_DATADIR "/Locations.xml");
-#else
-	filename = e_util_replace_prefix (E_DATA_SERVER_PREFIX,
-					  e_util_get_prefix (),
-					  WEATHER_DATADIR "/Locations.xml");
-#endif
+	ids = g_strsplit (code_name, "/", 2);
 
-	doc = e_xml_parse_file (filename);
+	if (!ids || !ids [0] || !ids [1])
+		goto done;
 
-	g_assert (doc != NULL);
+	model = gweather_xml_load_locations ();
+	if (!model)
+		goto done;
 
-	root = xmlDocGetRootElement (doc);
+	search.code = ids [0];
+	search.name = ids [1];
+	search.is_old = is_old;
+	search.location = NULL;
 
-	url = parse_for_url (sstation[0], sstation[1], root);
+	gtk_tree_model_foreach (model, (GtkTreeModelForeachFunc) find_location_func, &search);
 
-	source->url = g_strdup (url);
-	source->substation = g_strdup (sstation[0]);
+	gweather_xml_free_locations (model);
+	g_strfreev (ids);
 
-	g_strfreev (sstation);
+done:
+	return search.location;
 }
 
 EWeatherSource*
-e_weather_source_ccf_new (const char *uri)
+e_weather_source_ccf_new (const gchar *uri)
 {
-	/* Our URI is formatted as weather://ccf/AAA[/BBB] - AAA is the 3-letter station
+	/* Old URI is formatted as weather://ccf/AAA[/BBB] - AAA is the 3-letter station
 	 * code for identifying the providing station (subdirectory within the crh data
 	 * repository). BBB is an optional additional station ID for the station within
 	 * the CCF file. If not present, BBB is assumed to be the same station as AAA.
+	 * But the new URI is as weather://code/name, where code is 4-letter code.
+	 * So if got the old URI, then migrate to the new one, if possible.
 	 */
-	EWeatherSourceCCF *source = E_WEATHER_SOURCE_CCF (g_object_new (e_weather_source_ccf_get_type (), NULL));
 
-	find_station_url (strchr (uri, '/') + 1, source);
+	WeatherLocation *wl;
+	EWeatherSourceCCF *source;
+
+	if (!uri)
+		return NULL;
+
+	if (strncmp (uri, "ccf/", 4) == 0)
+		wl = find_location (uri + 4, TRUE);
+	else
+		wl = find_location (uri, FALSE);
+
+	if (!wl)
+		return NULL;
+
+	source = E_WEATHER_SOURCE_CCF (g_object_new (e_weather_source_ccf_get_type (), NULL));
+	source->location = wl;
+	source->info = NULL;
+
 	return E_WEATHER_SOURCE (source);
 }
 
+#if 0
 static GSList*
-tokenize (char *buffer)
+tokenize (gchar *buffer)
 {
-	char *token;
-	char *tokbuf;
+	gchar *token;
+	gchar *tokbuf;
 	GSList *ret;
 
 	token = strtok_r (buffer, " \n", &tokbuf);
@@ -153,9 +161,9 @@ tokenize (char *buffer)
 }
 
 static void
-date2tm (char *date, struct tm *times)
+date2tm (gchar *date, struct tm *times)
 {
-	char tmp[3];
+	gchar tmp[3];
 	time_t curtime = time(NULL);
 	tmp[2] = '\0';
 
@@ -170,7 +178,7 @@ date2tm (char *date, struct tm *times)
 }
 
 static WeatherConditions
-decodeConditions (char code)
+decodeConditions (gchar code)
 {
 	switch (code) {
 		case 'A': return WEATHER_FAIR;
@@ -204,10 +212,10 @@ decodeConditions (char code)
 	}
 }
 
-static int
-decodePOP (char data)
+static gint
+decodePOP (gchar data)
 {
-	int ret;
+	gint ret;
 
 	switch (data) {
 		case '-':
@@ -226,9 +234,9 @@ decodePOP (char data)
 }
 
 static void
-decodeSnowfall (char *data, float *low, float *high)
+decodeSnowfall (gchar *data, gfloat *low, gfloat *high)
 {
-	char num[3];
+	gchar num[3];
 	num[2] = '\0';
 
 	num[0] = data[0]; num[1] = data[1];
@@ -238,16 +246,16 @@ decodeSnowfall (char *data, float *low, float *high)
 }
 
 static float
-ftoc (char *data)
+ftoc (gchar *data)
 {
-	int fahrenheit = atoi(data);
+	gint fahrenheit = atoi(data);
 	if (fahrenheit >= 900)
 		fahrenheit = (fahrenheit - 900) * -1;
-	return ((float)(fahrenheit-32)) * 5.0f / 9.0f;
+	return ((gfloat)(fahrenheit-32)) * 5.0f / 9.0f;
 }
 
 static void
-e_weather_source_ccf_do_parse (EWeatherSourceCCF *source, char *buffer)
+e_weather_source_ccf_do_parse (EWeatherSourceCCF *source, gchar *buffer)
 {
 	/* CCF gives us either 2 or 7 days of forecast data. IFPS WFO's
 	 * will produce 7 day forecasts, whereas pre-IFPS WFO's are only
@@ -265,7 +273,7 @@ e_weather_source_ccf_do_parse (EWeatherSourceCCF *source, char *buffer)
 	GSList *current = tokens;
 	GList *fc = NULL;
 	struct tm tms;
-	int i;
+	gint i;
 	time_t base;
 	gint n;
 
@@ -278,8 +286,8 @@ e_weather_source_ccf_do_parse (EWeatherSourceCCF *source, char *buffer)
 		current = g_slist_next (current);
 	current = g_slist_next (current);
 	/* pick up the first two conditions reports */
-	forecasts[0].conditions = decodeConditions (((char*)(current->data))[0]);
-	forecasts[1].conditions = decodeConditions (((char*)(current->data))[1]);
+	forecasts[0].conditions = decodeConditions (((gchar *)(current->data))[0]);
+	forecasts[1].conditions = decodeConditions (((gchar *)(current->data))[1]);
 
 	current = g_slist_next (current);
 	if (tms.tm_hour < 12) {
@@ -291,8 +299,8 @@ e_weather_source_ccf_do_parse (EWeatherSourceCCF *source, char *buffer)
 		}
 		forecasts[2].high = ftoc (current->data);
 		current = g_slist_next (current);
-		forecasts[0].pop = decodePOP (((char*)(current->data))[2]);
-		forecasts[1].pop = decodePOP (((char*)(current->data))[4]);
+		forecasts[0].pop = decodePOP (((gchar *)(current->data))[2]);
+		forecasts[1].pop = decodePOP (((gchar *)(current->data))[4]);
 	} else {
 		for (i = 0; i < 2; i++) {
 			current = g_slist_next (current);
@@ -301,8 +309,8 @@ e_weather_source_ccf_do_parse (EWeatherSourceCCF *source, char *buffer)
 			forecasts[i].low  = ftoc (current->data);
 		}
 		current = g_slist_next (current);
-		forecasts[0].pop = decodePOP (((char*)(current->data))[1]);
-		forecasts[1].pop = decodePOP (((char*)(current->data))[3]);
+		forecasts[0].pop = decodePOP (((gchar *)(current->data))[1]);
+		forecasts[1].pop = decodePOP (((gchar *)(current->data))[3]);
 	}
 
 	current = g_slist_next (current);
@@ -336,11 +344,11 @@ e_weather_source_ccf_do_parse (EWeatherSourceCCF *source, char *buffer)
 	}
 
 	/* Grab the conditions for the next 5 days */
-	forecasts[2].conditions = decodeConditions (((char*)(current->data))[0]);
-	forecasts[3].conditions = decodeConditions (((char*)(current->data))[1]);
-	forecasts[4].conditions = decodeConditions (((char*)(current->data))[2]);
-	forecasts[5].conditions = decodeConditions (((char*)(current->data))[3]);
-	forecasts[6].conditions = decodeConditions (((char*)(current->data))[4]);
+	forecasts[2].conditions = decodeConditions (((gchar *)(current->data))[0]);
+	forecasts[3].conditions = decodeConditions (((gchar *)(current->data))[1]);
+	forecasts[4].conditions = decodeConditions (((gchar *)(current->data))[2]);
+	forecasts[5].conditions = decodeConditions (((gchar *)(current->data))[3]);
+	forecasts[6].conditions = decodeConditions (((gchar *)(current->data))[4]);
 
 	/* Temperature forecasts */
 	current = g_slist_next (current);
@@ -356,11 +364,11 @@ e_weather_source_ccf_do_parse (EWeatherSourceCCF *source, char *buffer)
 		forecasts[6].high = ftoc (current->data);
 		forecasts[6].low  = forecasts[6].high;
 		current = g_slist_next (current);
-		forecasts[2].pop = decodePOP (((char*)(current->data))[1]);
-		forecasts[3].pop = decodePOP (((char*)(current->data))[3]);
-		forecasts[4].pop = decodePOP (((char*)(current->data))[5]);
-		forecasts[5].pop = decodePOP (((char*)(current->data))[7]);
-		forecasts[6].pop = decodePOP (((char*)(current->data))[9]);
+		forecasts[2].pop = decodePOP (((gchar *)(current->data))[1]);
+		forecasts[3].pop = decodePOP (((gchar *)(current->data))[3]);
+		forecasts[4].pop = decodePOP (((gchar *)(current->data))[5]);
+		forecasts[5].pop = decodePOP (((gchar *)(current->data))[7]);
+		forecasts[6].pop = decodePOP (((gchar *)(current->data))[9]);
 		n = 7;
 	} else {
 		for (i = 2; i < 6; i++) {
@@ -373,11 +381,11 @@ e_weather_source_ccf_do_parse (EWeatherSourceCCF *source, char *buffer)
 		/* hack for people who put out bad data, like Pueblo, CO. Yes, PUB, that means you */
 		if (strlen (current->data) == 3)
 			current = g_slist_next (current);
-		forecasts[1].pop = decodePOP (((char*)(current->data))[0]);
-		forecasts[2].pop = decodePOP (((char*)(current->data))[2]);
-		forecasts[3].pop = decodePOP (((char*)(current->data))[4]);
-		forecasts[4].pop = decodePOP (((char*)(current->data))[6]);
-		forecasts[5].pop = decodePOP (((char*)(current->data))[8]);
+		forecasts[1].pop = decodePOP (((gchar *)(current->data))[0]);
+		forecasts[2].pop = decodePOP (((gchar *)(current->data))[2]);
+		forecasts[3].pop = decodePOP (((gchar *)(current->data))[4]);
+		forecasts[4].pop = decodePOP (((gchar *)(current->data))[6]);
+		forecasts[5].pop = decodePOP (((gchar *)(current->data))[8]);
 	}
 
 	for (i = 0; i < n; i++) {
@@ -388,74 +396,48 @@ e_weather_source_ccf_do_parse (EWeatherSourceCCF *source, char *buffer)
 	g_free (forecasts);
 	g_list_free (fc);
 }
+#endif
 
 static void
-retrieval_done (SoupSession *session, SoupMessage *message, EWeatherSourceCCF *source)
+parse_done (WeatherInfo *info, gpointer data)
 {
-	/* check status code */
-	if (!SOUP_STATUS_IS_SUCCESSFUL (message->status_code)) {
-		source->done (NULL, source->finished_data);
+	EWeatherSourceCCF *ccfsource = (EWeatherSourceCCF *) data;
+
+	if (!ccfsource)
+		return;
+
+	if (!info || !weather_info_is_valid (info)) {
+		ccfsource->done (NULL, ccfsource->finished_data);
 		return;
 	}
 
-	e_weather_source_ccf_do_parse (source, (char *)message->response_body->data);
+	ccfsource->done (info, ccfsource->finished_data);
 }
 
 static void
 e_weather_source_ccf_parse (EWeatherSource *source, EWeatherSourceFinished done, gpointer data)
 {
 	EWeatherSourceCCF *ccfsource = (EWeatherSourceCCF*) source;
-	SoupMessage *soup_message;
+	WeatherPrefs prefs;
 
 	ccfsource->finished_data = data;
-
 	ccfsource->done = done;
 
-	if (!ccfsource->soup_session) {
-		GConfClient *conf_client;
-		ccfsource->soup_session = soup_session_async_new ();
+	prefs.type = FORECAST_LIST;
+	prefs.radar = FALSE;
+	prefs.radar_custom_url = NULL;
+	prefs.temperature_unit = TEMP_UNIT_CENTIGRADE;
+	prefs.speed_unit = SPEED_UNIT_MS;
+	prefs.pressure_unit = PRESSURE_UNIT_HPA;
+	prefs.distance_unit = DISTANCE_UNIT_METERS;
 
-		/* set the HTTP proxy, if configuration is set to do so */
-		conf_client = gconf_client_get_default ();
-		if (gconf_client_get_bool (conf_client, "/system/http_proxy/use_http_proxy", NULL)) {
-			char *server, *proxy_uri;
-			int port;
-
-			server = gconf_client_get_string (conf_client, "/system/http_proxy/host", NULL);
-			port = gconf_client_get_int (conf_client, "/system/http_proxy/port", NULL);
-
-			if (server && server[0]) {
-				SoupURI *suri;
-				if (gconf_client_get_bool (conf_client, "/system/http_proxy/use_authentication", NULL)) {
-					char *user, *password;
-
-					user = gconf_client_get_string (conf_client,
-									"/system/http_proxy/authentication_user",
-									NULL);
-					password = gconf_client_get_string (conf_client,
-									    "/system/http_proxy/authentication_password",
-									    NULL);
-
-					proxy_uri = g_strdup_printf("http://%s:%s@%s:%d", user, password, server, port);
-
-					g_free (user);
-					g_free (password);
-				} else
-					proxy_uri = g_strdup_printf ("http://%s:%d", server, port);
-
-				suri = soup_uri_new (proxy_uri);
-				g_object_set (G_OBJECT (ccfsource->soup_session), SOUP_SESSION_PROXY_URI, suri, NULL);
-
-				soup_uri_free (suri);
-				g_free (server);
-				g_free (proxy_uri);
-			}
-		}
-		g_object_unref (conf_client);
+	if (ccfsource->location && !ccfsource->info) {
+		ccfsource->info = weather_info_new (ccfsource->location, &prefs, parse_done, source);
+		weather_location_free (ccfsource->location);
+		ccfsource->location = NULL;
+	} else {
+		ccfsource->info = weather_info_update (ccfsource->info, &prefs, parse_done, source);
 	}
-
-	soup_message = soup_message_new (SOUP_METHOD_GET, ccfsource->url);
-	soup_session_queue_message (ccfsource->soup_session, soup_message, (SoupSessionCallback) retrieval_done, source);
 }
 
 static void
@@ -471,9 +453,8 @@ e_weather_source_ccf_class_init (EWeatherSourceCCFClass *class)
 static void
 e_weather_source_ccf_init (EWeatherSourceCCF *source)
 {
-	source->url = NULL;
-	source->substation = NULL;
-	source->soup_session = NULL;
+	source->location = NULL;
+	source->info = NULL;
 }
 
 GType
