@@ -1,7 +1,6 @@
 /* Evolution calendar utilities and types
  *
- * Copyright (C) 2000 Ximian, Inc.
- * Copyright (C) 2000 Ximian, Inc.
+ * Copyright (C) 1999-2008 Novell, Inc. (www.novell.com)
  *
  * Author: Federico Mena-Quintero <federico@ximian.com>
  *
@@ -27,6 +26,7 @@
 #include <glib/gi18n-lib.h>
 #include "libedataserver/e-data-server-util.h"
 #include "e-cal-util.h"
+#include "e-cal-system-timezone.h"
 
 
 
@@ -67,7 +67,7 @@ cal_obj_uid_list_free (GList *list)
 	GList *l;
 
 	for (l = list; l; l = l->next) {
-		char *uid;
+		gchar *uid;
 
 		uid = l->data;
 
@@ -123,19 +123,22 @@ e_cal_util_new_component (icalcomponent_kind kind)
 {
 	icalcomponent *comp;
 	struct icaltimetype dtstamp;
+	gchar *uid;
 
 	comp = icalcomponent_new (kind);
-	icalcomponent_set_uid (comp, e_cal_component_gen_uid ());
+	uid = e_cal_component_gen_uid ();
+	icalcomponent_set_uid (comp, uid);
+	g_free (uid);
 	dtstamp = icaltime_current_time_with_zone (icaltimezone_get_utc_timezone ());
 	icalcomponent_set_dtstamp (comp, dtstamp);
 
 	return comp;
 }
 
-static char *
-read_line (const char *string)
+static gchar *
+read_line (const gchar *string)
 {
-	char *line;
+	gchar *line;
 	GString *line_str = NULL;
 
 	for (; *string; string++) {
@@ -157,17 +160,18 @@ read_line (const char *string)
  * e_cal_util_parse_ics_string:
  * @string: iCalendar string to be parsed.
  *
- * Parses an iCalendar stirng and returns an #icalcomponent representing that
- * string. Note that this function deals with multiple VCALENDAR's in the
+ * Parses an iCalendar string and returns a new #icalcomponent representing
+ * that string. Note that this function deals with multiple VCALENDAR's in the
  * string, something that Mozilla used to do and which libical does not
  * support.
  *
- * Return value: an #icalcomponent.
+ * Return value: a newly created #icalcomponent or NULL if the string isn't a
+ * valid iCalendar string.
  */
 icalcomponent *
-e_cal_util_parse_ics_string (const char *string)
+e_cal_util_parse_ics_string (const gchar *string)
 {
-	char *s;
+	gchar *s;
 	icalcomponent *icalcomp = NULL;
 
 	g_return_val_if_fail (string != NULL, NULL);
@@ -177,7 +181,7 @@ e_cal_util_parse_ics_string (const char *string)
 		GString *comp_str = NULL;
 
 		while (*s) {
-			char *line = read_line (s);
+			gchar *line = read_line (s);
 			if (line) {
 				if (!comp_str)
 					comp_str = g_string_new (line);
@@ -211,8 +215,8 @@ e_cal_util_parse_ics_string (const char *string)
 	return icalcomp;
 }
 
-static char *
-get_line_fn (char *buf, size_t size, void *file)
+static gchar *
+get_line_fn (gchar *buf, gsize size, gpointer file)
 {
 	return fgets (buf, size, file);
 }
@@ -222,12 +226,13 @@ get_line_fn (char *buf, size_t size, void *file)
  * @filename: Name of the file to be parsed.
  *
  * Parses the given file, and, if it contains a valid iCalendar object,
- * parse it and return a corresponding #icalcomponent.
+ * parse it and return a new #icalcomponent.
  *
- * Return value: an #icalcomponent.
+ * Return value: a newly created #icalcomponent or NULL if the file doesn't
+ * contain a valid iCalendar object.
  */
 icalcomponent *
-e_cal_util_parse_ics_file (const char *filename)
+e_cal_util_parse_ics_file (const gchar *filename)
 {
 	icalparser *parser;
 	icalcomponent *icalcomp;
@@ -263,7 +268,7 @@ compute_alarm_range (ECalComponent *comp, GList *alarm_uids, time_t start, time_
 	repeat_time = 0;
 
 	for (l = alarm_uids; l; l = l->next) {
-		const char *auid;
+		const gchar *auid;
 		ECalComponentAlarm *alarm;
 		ECalComponentAlarmTrigger trigger;
 		struct icaldurationtype *dur;
@@ -286,10 +291,10 @@ compute_alarm_range (ECalComponent *comp, GList *alarm_uids, time_t start, time_
 		case E_CAL_COMPONENT_ALARM_TRIGGER_RELATIVE_START:
 		case E_CAL_COMPONENT_ALARM_TRIGGER_RELATIVE_END:
 			dur = &trigger.u.rel_duration;
-  			dur_time = icaldurationtype_as_int (*dur);
+			dur_time = icaldurationtype_as_int (*dur);
 
 			if (repeat.repetitions != 0) {
-				int rdur;
+				gint rdur;
 
 				rdur = repeat.repetitions * icaldurationtype_as_int (repeat.duration);
 				repeat_time = MAX (repeat_time, rdur);
@@ -326,11 +331,11 @@ struct alarm_occurrence_data {
 
 	/* This is what we compute */
 	GSList *triggers;
-	int n_triggers;
+	gint n_triggers;
 };
 
 static void
-add_trigger (struct alarm_occurrence_data *aod, const char *auid, time_t trigger,
+add_trigger (struct alarm_occurrence_data *aod, const gchar *auid, time_t trigger,
 	     time_t occur_start, time_t occur_end)
 {
 	ECalComponentAlarmInstance *instance;
@@ -357,7 +362,7 @@ add_alarm_occurrences_cb (ECalComponent *comp, time_t start, time_t end, gpointe
 	aod = data;
 
 	for (l = aod->alarm_uids; l; l = l->next) {
-		const char *auid;
+		const gchar *auid;
 		ECalComponentAlarm *alarm;
 		ECalComponentAlarmAction action;
 		ECalComponentAlarmTrigger trigger;
@@ -365,7 +370,7 @@ add_alarm_occurrences_cb (ECalComponent *comp, time_t start, time_t end, gpointe
 		struct icaldurationtype *dur;
 		time_t dur_time;
 		time_t occur_time, trigger_time;
-		int i;
+		gint i;
 
 		auid = l->data;
 		alarm = e_cal_component_get_alarm (comp, auid);
@@ -388,7 +393,7 @@ add_alarm_occurrences_cb (ECalComponent *comp, time_t start, time_t end, gpointe
 			continue;
 
 		dur = &trigger.u.rel_duration;
-  		dur_time = icaldurationtype_as_int (*dur);
+		dur_time = icaldurationtype_as_int (*dur);
 
 		if (trigger.type == E_CAL_COMPONENT_ALARM_TRIGGER_RELATIVE_START)
 			occur_time = start;
@@ -406,7 +411,7 @@ add_alarm_occurrences_cb (ECalComponent *comp, time_t start, time_t end, gpointe
 		/* Add repeating alarms */
 
 		if (repeat.repetitions != 0) {
-			int i;
+			gint i;
 			time_t repeat_time;
 
 			repeat_time = icaldurationtype_as_int (repeat.duration);
@@ -444,7 +449,7 @@ generate_absolute_triggers (ECalComponent *comp, struct alarm_occurrence_data *a
 	e_cal_component_get_dtend (comp, &dt_end);
 
 	for (l = aod->alarm_uids; l; l = l->next) {
-		const char *auid;
+		const gchar *auid;
 		ECalComponentAlarm *alarm;
 		ECalComponentAlarmAction action;
 		ECalComponentAlarmRepeat repeat;
@@ -452,7 +457,7 @@ generate_absolute_triggers (ECalComponent *comp, struct alarm_occurrence_data *a
 		time_t abs_time;
 		time_t occur_start, occur_end;
 		icaltimezone *zone;
-		int i;
+		gint i;
 
 		auid = l->data;
 		alarm = e_cal_component_get_alarm (comp, auid);
@@ -503,7 +508,7 @@ generate_absolute_triggers (ECalComponent *comp, struct alarm_occurrence_data *a
 		/* Add repeating alarms */
 
 		if (repeat.repetitions != 0) {
-			int i;
+			gint i;
 			time_t repeat_time;
 
 			repeat_time = icaldurationtype_as_int (repeat.duration);
@@ -631,7 +636,7 @@ e_cal_util_generate_alarms_for_comp (ECalComponent *comp,
  *
  * Return value: the number of elements it added to the list.
  */
-int
+gint
 e_cal_util_generate_alarms_for_list (GList *comps,
 				     time_t start,
 				     time_t end,
@@ -642,7 +647,7 @@ e_cal_util_generate_alarms_for_list (GList *comps,
 				     icaltimezone *default_timezone)
 {
 	GList *l;
-	int n;
+	gint n;
 
 	n = 0;
 
@@ -662,7 +667,6 @@ e_cal_util_generate_alarms_for_list (GList *comps,
 	return n;
 }
 
-
 /**
  * e_cal_util_priority_to_string:
  * @priority: Priority value.
@@ -673,10 +677,10 @@ e_cal_util_generate_alarms_for_list (GList *comps,
  * Return value: a string representing the PRIORITY value. This value is a
  * constant, so it should never be freed.
  */
-char *
-e_cal_util_priority_to_string (int priority)
+const gchar *
+e_cal_util_priority_to_string (gint priority)
 {
-	char *retval;
+	const gchar *retval;
 
 	if (priority <= 0)
 		retval = "";
@@ -692,7 +696,6 @@ e_cal_util_priority_to_string (int priority)
 	return retval;
 }
 
-
 /**
  * e_cal_util_priority_from_string:
  * @string: A string representing the PRIORITY value.
@@ -701,10 +704,10 @@ e_cal_util_priority_to_string (int priority)
  *
  * Return value: the priority (0-9) or -1 if the priority string is not valid.
 */
-int
-e_cal_util_priority_from_string (const char *string)
+gint
+e_cal_util_priority_from_string (const gchar *string)
 {
-	int priority;
+	gint priority;
 
 	/* An empty string is the same as 'None'. */
 	if (!string || !string[0] || !e_util_utf8_strcasecmp (string, _("Undefined")))
@@ -728,10 +731,10 @@ typedef struct {
 } ForeachTzidData;
 
 static void
-add_timezone_cb (icalparameter *param, void *data)
+add_timezone_cb (icalparameter *param, gpointer data)
 {
 	icaltimezone *tz;
-	const char *tzid;
+	const gchar *tzid;
 	icalcomponent *vtz_comp;
 	ForeachTzidData *f_data = (ForeachTzidData *) data;
 
@@ -950,8 +953,6 @@ e_cal_util_event_dates_match (icalcomponent *icalcomp1, icalcomponent *icalcomp2
 			return FALSE;
 	}
 
-
-
 	/* now match the timezones */
 	if (!(!c1_dtstart.zone && !c2_dtstart.zone) ||
 	    (c1_dtstart.zone && c2_dtstart.zone &&
@@ -976,7 +977,7 @@ struct instance_data {
 };
 
 static void
-check_instance (icalcomponent *comp, struct icaltime_span *span, void *data)
+check_instance (icalcomponent *comp, struct icaltime_span *span, gpointer data)
 {
 	struct instance_data *instance = data;
 
@@ -1030,7 +1031,7 @@ e_cal_util_construct_instance (icalcomponent *icalcomp,
 static inline gboolean
 time_matches_rid (struct icaltimetype itt, struct icaltimetype rid, CalObjModType mod)
 {
-	int compare;
+	gint compare;
 
 	compare = icaltime_compare (itt, rid);
 	if (compare == 0)
@@ -1138,3 +1139,37 @@ e_cal_util_remove_instances (icalcomponent *icalcomp,
 		icalrecur_iterator_free (iter);
 	}
 }
+
+/**
+ * e_cal_util_get_system_timezone_location:
+ *
+ * Returns system timezone location string, NULL on an error.
+ * Returned pointer should be freed with g_free().
+ **/
+gchar *
+e_cal_util_get_system_timezone_location (void)
+{
+	return e_cal_system_timezone_get_location ();
+}
+
+/**
+ * e_cal_util_get_system_timezone:
+ *
+ * Returns icaltimezone object of the system timezone. NULL on an error.
+ * Returned pointer is part of the built-in timezones, thus do not free it.
+ **/
+icaltimezone *
+e_cal_util_get_system_timezone (void)
+{
+	gchar *location;
+	icaltimezone *zone;
+
+	location = e_cal_system_timezone_get_location ();
+	g_return_val_if_fail (location != NULL, NULL);
+
+	zone = icaltimezone_get_builtin_timezone (location);
+
+	g_free (location);
+
+	return zone;
+}
