@@ -30,8 +30,6 @@
 
 #include <string.h>
 
-#include <libedataserver/e-sexp.h>
-
 #include "camel-mime-utils.h"	/* base64 encoding */
 #include "camel-search-private.h"
 #include "camel-seekable-stream.h"
@@ -83,19 +81,18 @@ struct _match_record {
 	struct _match_record *next;
 	struct _match_record *prev;
 
-	char hash[17];
+	gchar hash[17];
 
 	guint32 lastuid;
 	guint32 validity;
 
-	unsigned int termcount;
-	char **terms;
+	guint termcount;
+	gchar **terms;
 	GArray *matches;
 };
 
-
 static void free_match(CamelImapSearch *is, struct _match_record *mr);
-static ESExpResult *imap_body_contains (struct _ESExp *f, int argc, struct _ESExpResult **argv, CamelFolderSearch *s);
+static ESExpResult *imap_body_contains (struct _ESExp *f, gint argc, struct _ESExpResult **argv, CamelFolderSearch *s);
 
 static CamelFolderSearchClass *imap_search_parent_class;
 
@@ -107,7 +104,7 @@ camel_imap_search_class_init (CamelImapSearchClass *camel_imap_search_class)
 		CAMEL_FOLDER_SEARCH_CLASS (camel_imap_search_class);
 
 	imap_search_parent_class = (CamelFolderSearchClass *)camel_type_get_global_classfuncs (camel_folder_search_get_type ());
-	
+
 	/* virtual method overload */
 	camel_folder_search_class->body_contains = imap_body_contains;
 }
@@ -115,7 +112,7 @@ camel_imap_search_class_init (CamelImapSearchClass *camel_imap_search_class)
 static void
 camel_imap_search_init(CamelImapSearch *is)
 {
-	e_dlist_init(&is->matches);
+	camel_dlist_init(&is->matches);
 	is->matches_hash = g_hash_table_new(g_str_hash, g_str_equal);
 	is->matches_count = 0;
 	is->lastuid = 0;
@@ -126,7 +123,7 @@ camel_imap_search_finalise(CamelImapSearch *is)
 {
 	struct _match_record *mr;
 
-	while ( (mr = (struct _match_record *)e_dlist_remtail(&is->matches)) )
+	while ( (mr = (struct _match_record *)camel_dlist_remtail(&is->matches)) )
 		free_match(is, mr);
 	g_hash_table_destroy(is->matches_hash);
 	if (is->cache)
@@ -137,7 +134,7 @@ CamelType
 camel_imap_search_get_type (void)
 {
 	static CamelType camel_imap_search_type = CAMEL_INVALID_TYPE;
-	
+
 	if (camel_imap_search_type == CAMEL_INVALID_TYPE) {
 		camel_imap_search_type = camel_type_register (
 			CAMEL_FOLDER_SEARCH_TYPE, "CamelImapSearch",
@@ -157,7 +154,7 @@ camel_imap_search_get_type (void)
  * Return value: A new CamelImapSearch widget.
  **/
 CamelFolderSearch *
-camel_imap_search_new (const char *cachedir)
+camel_imap_search_new (const gchar *cachedir)
 {
 	CamelFolderSearch *new = CAMEL_FOLDER_SEARCH (camel_object_new (camel_imap_search_get_type ()));
 	CamelImapSearch *is = (CamelImapSearch *)new;
@@ -173,15 +170,14 @@ camel_imap_search_new (const char *cachedir)
 	return new;
 }
 
-
 static void
-hash_match(char hash[17], int argc, struct _ESExpResult **argv)
+hash_match(gchar hash[17], gint argc, struct _ESExpResult **argv)
 {
 	GChecksum *checksum;
 	guint8 *digest;
 	gsize length;
-	int state = 0, save = 0;
-	int i;
+	gint state = 0, save = 0;
+	gint i;
 
 	length = g_checksum_type_get_length (G_CHECKSUM_MD5);
 	digest = g_alloca (length);
@@ -208,11 +204,11 @@ hash_match(char hash[17], int argc, struct _ESExpResult **argv)
 	hash[16] = 0;
 }
 
-static int
+static gint
 save_match(CamelImapSearch *is, struct _match_record *mr)
 {
 	guint32 mark = MATCH_MARK;
-	int ret = 0;
+	gint ret = 0;
 	struct _match_header header;
 	CamelStream *stream;
 
@@ -226,7 +222,7 @@ save_match(CamelImapSearch *is, struct _match_record *mr)
 		return -1;
 
 	d(printf("Saving search cache entry to '%s': %s\n", mr->hash, mr->terms[0]));
-	
+
 	/* we write the whole thing, then re-write the header magic, saves fancy sync code */
 	memcpy(&header.mark, "    ", 4);
 	header.termcount = 0;
@@ -234,10 +230,10 @@ save_match(CamelImapSearch *is, struct _match_record *mr)
 	header.lastuid = mr->lastuid;
 	header.validity = mr->validity;
 
-	if (camel_stream_write(stream, (char *)&header, sizeof(header)) != sizeof(header)
+	if (camel_stream_write(stream, (gchar *)&header, sizeof(header)) != sizeof(header)
 	    || camel_stream_write(stream, mr->matches->data, mr->matches->len*sizeof(guint32)) != mr->matches->len*sizeof(guint32)
 	    || camel_seekable_stream_seek((CamelSeekableStream *)stream, 0, CAMEL_STREAM_SET) == -1
-	    || camel_stream_write(stream, (char *)&mark, sizeof(mark)) != sizeof(mark)) {
+	    || camel_stream_write(stream, (gchar *)&mark, sizeof(mark)) != sizeof(mark)) {
 		d(printf(" saving failed, removing cache entry\n"));
 		camel_data_cache_remove(is->cache, "search/body-contains", mr->hash, NULL);
 		ret = -1;
@@ -250,7 +246,7 @@ save_match(CamelImapSearch *is, struct _match_record *mr)
 static void
 free_match(CamelImapSearch *is, struct _match_record *mr)
 {
-	int i;
+	gint i;
 
 	for (i=0;i<mr->termcount;i++)
 		g_free(mr->terms[i]);
@@ -260,12 +256,12 @@ free_match(CamelImapSearch *is, struct _match_record *mr)
 }
 
 static struct _match_record *
-load_match(CamelImapSearch *is, char hash[17], int argc, struct _ESExpResult **argv)
+load_match(CamelImapSearch *is, gchar hash[17], gint argc, struct _ESExpResult **argv)
 {
 	struct _match_record *mr;
 	CamelStream *stream = NULL;
 	struct _match_header header;
-	int i;
+	gint i;
 
 	mr = g_malloc0(sizeof(*mr));
 	mr->matches = g_array_new(0, 0, sizeof(guint32));
@@ -290,7 +286,7 @@ load_match(CamelImapSearch *is, char hash[17], int argc, struct _ESExpResult **a
 		   should be sufficient to key it */
 		/* This check should also handle endianness changes, we just throw away
 		   the data (its only a cache) */
-		if (camel_stream_read(stream, (char *)&header, sizeof(header)) == sizeof(header)
+		if (camel_stream_read(stream, (gchar *)&header, sizeof(header)) == sizeof(header)
 		    && header.validity == is->validity
 		    && header.mark == MATCH_MARK
 		    && header.termcount == 0) {
@@ -315,17 +311,17 @@ load_match(CamelImapSearch *is, char hash[17], int argc, struct _ESExpResult **a
 	return mr;
 }
 
-static int
+static gint
 sync_match(CamelImapSearch *is, struct _match_record *mr)
 {
-	char *p, *result, *lasts = NULL;
+	gchar *p, *result, *lasts = NULL;
 	CamelImapResponse *response = NULL;
 	guint32 uid;
 	CamelFolder *folder = ((CamelFolderSearch *)is)->folder;
 	CamelImapStore *store = (CamelImapStore *)folder->parent_store;
 	struct _camel_search_words *words;
 	GString *search;
-	int i;
+	gint i;
 
 	if (mr->lastuid >= is->lastuid && mr->validity == is->validity)
 		return 0;
@@ -335,11 +331,11 @@ sync_match(CamelImapSearch *is, struct _match_record *mr)
 	/* TODO: Handle multiple search terms */
 
 	/* This handles multiple search words within a single term */
-	words = camel_search_words_split ((const unsigned char *) mr->terms[0]);
+	words = camel_search_words_split ((const guchar *) mr->terms[0]);
 	search = g_string_new ("");
 	g_string_append_printf (search, "UID %d:%d", mr->lastuid + 1, is->lastuid);
 	for (i = 0; i < words->len; i++) {
-		char *w = words->words[i]->word, c;
+		gchar *w = words->words[i]->word, c;
 
 		g_string_append_printf (search, " BODY \"");
 		while ((c = *w++)) {
@@ -350,7 +346,7 @@ sync_match(CamelImapSearch *is, struct _match_record *mr)
 		g_string_append_c (search, '"');
 	}
 	camel_search_words_free (words);
-	
+
 	/* We only try search using utf8 if its non us-ascii text? */
 	if ((words->type & CAMEL_SEARCH_WORD_8BIT) &&  (store->capabilities & IMAP_CAPABILITY_utf8_search)) {
 		response = camel_imap_command (store, folder, NULL,
@@ -369,7 +365,7 @@ sync_match(CamelImapSearch *is, struct _match_record *mr)
 	result = camel_imap_response_extract (store, response, "SEARCH", NULL);
 	if (!result)
 		return -1;
-	
+
 	p = result + sizeof ("* SEARCH");
 	for (p = strtok_r (p, " ", &lasts); p; p = strtok_r (NULL, " ", &lasts)) {
 		uid = strtoul(p, NULL, 10);
@@ -385,9 +381,9 @@ sync_match(CamelImapSearch *is, struct _match_record *mr)
 }
 
 static struct _match_record *
-get_match(CamelImapSearch *is, int argc, struct _ESExpResult **argv)
+get_match(CamelImapSearch *is, gint argc, struct _ESExpResult **argv)
 {
-	char hash[17];
+	gchar hash[17];
 	struct _match_record *mr;
 
 	hash_match(hash, argc, argv);
@@ -395,7 +391,7 @@ get_match(CamelImapSearch *is, int argc, struct _ESExpResult **argv)
 	mr = g_hash_table_lookup(is->matches_hash, hash);
 	if (mr == NULL) {
 		while (is->matches_count >= MATCH_CACHE_SIZE) {
-			mr = (struct _match_record *)e_dlist_remtail(&is->matches);
+			mr = (struct _match_record *)camel_dlist_remtail(&is->matches);
 			if (mr) {
 				printf("expiring match '%s' (%s)\n", mr->hash, mr->terms[0]);
 				g_hash_table_remove(is->matches_hash, mr->hash);
@@ -409,10 +405,10 @@ get_match(CamelImapSearch *is, int argc, struct _ESExpResult **argv)
 		g_hash_table_insert(is->matches_hash, mr->hash, mr);
 		is->matches_count++;
 	} else {
-		e_dlist_remove((EDListNode *)mr);
+		camel_dlist_remove((CamelDListNode *)mr);
 	}
 
-	e_dlist_addhead(&is->matches, (EDListNode *)mr);
+	camel_dlist_addhead(&is->matches, (CamelDListNode *)mr);
 
 	/* what about offline mode? */
 	/* We could cache those results too, or should we cache them elsewhere? */
@@ -422,16 +418,15 @@ get_match(CamelImapSearch *is, int argc, struct _ESExpResult **argv)
 }
 
 static ESExpResult *
-imap_body_contains (struct _ESExp *f, int argc, struct _ESExpResult **argv, CamelFolderSearch *s)
+imap_body_contains (struct _ESExp *f, gint argc, struct _ESExpResult **argv, CamelFolderSearch *s)
 {
 	CamelImapStore *store = CAMEL_IMAP_STORE (s->folder->parent_store);
 	CamelImapSearch *is = (CamelImapSearch *)s;
-	char *uid;
+	gchar *uid;
 	ESExpResult *r;
-	CamelMessageInfo *info;	
 	GHashTable *uid_hash = NULL;
 	GPtrArray *array;
-	int i, j;
+	gint i, j;
 	struct _match_record *mr;
 	guint32 uidn, *uidp;
 
@@ -440,7 +435,7 @@ imap_body_contains (struct _ESExp *f, int argc, struct _ESExpResult **argv, Came
 	/* TODO: Cache offline searches too? */
 
 	/* If offline, search using the parent class, which can handle this manually */
-	if (!camel_disco_store_check_online (CAMEL_DISCO_STORE (store), NULL))
+	if (CAMEL_OFFLINE_STORE (store)->state == CAMEL_OFFLINE_STORE_NETWORK_UNAVAIL)
 		return imap_search_parent_class->body_contains(f, argc, argv, s);
 
 	/* optimise the match "" case - match everything */
@@ -452,8 +447,7 @@ imap_body_contains (struct _ESExp *f, int argc, struct _ESExpResult **argv, Came
 			r = e_sexp_result_new(f, ESEXP_RES_ARRAY_PTR);
 			r->value.ptrarray = g_ptr_array_new ();
 			for (i = 0; i < s->summary->len; i++) {
-				info = g_ptr_array_index(s->summary, i);
-				g_ptr_array_add(r->value.ptrarray, (char *)camel_message_info_uid(info));
+				g_ptr_array_add(r->value.ptrarray, (gchar *)g_ptr_array_index(s->summary, i));
 			}
 		}
 	} else if (argc == 0 || s->summary->len == 0) {
@@ -466,11 +460,10 @@ imap_body_contains (struct _ESExp *f, int argc, struct _ESExpResult **argv, Came
 			r->value.ptrarray = g_ptr_array_new ();
 		}
 	} else {
-		int truth = FALSE;
+		gint truth = FALSE;
 
 		/* setup lastuid/validity for synchronising */
-		info = g_ptr_array_index(s->summary, s->summary->len-1);
-		is->lastuid = strtoul(camel_message_info_uid(info), NULL, 10);
+		is->lastuid = strtoul((gchar *)g_ptr_array_index(s->summary, s->summary->len-1), NULL, 10);
 		is->validity = ((CamelImapSummary *)(s->folder->summary))->validity;
 
 		mr = get_match(is, argc, argv);
@@ -491,8 +484,7 @@ imap_body_contains (struct _ESExp *f, int argc, struct _ESExpResult **argv, Came
 			/* We use the summary's strings so we dont need to alloc more */
 			uid_hash = g_hash_table_new(NULL, NULL);
 			for (i = 0; i < s->summary->len; i++) {
-				info = s->summary->pdata[i];
-				uid = (char *)camel_message_info_uid(info);
+				uid = (gchar *)s->summary->pdata[i];
 				uidn = strtoul(uid, NULL, 10);
 				g_hash_table_insert(uid_hash, GUINT_TO_POINTER(uidn), uid);
 			}
